#include <Wire.h>
#include <LiquidCrystal.h>
#include <Servo.h>
#include "HX711.h"

// Define pins
#define TRIG_PIN_ULTRA1 24
#define ECHO_PIN_ULTRA1 25
#define RED_LED_PIN 23
#define GREEN_LED_PIN 22
#define SERVO1_PIN A1
#define SERVO2_PIN A2
#define BUZZER_PIN 26

// Define weight threshold
const float weightThreshold = 1000; // 1kg in grams

// Object Defining
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);
Servo servo1, servo2;
HX711 scale;

// Timing
long duration1, distance1;
bool weightOk = false; // Flag to track if the weight is below the weightThreshold

void setup() {
  // Initialize serial communication
  Serial.begin(9600);

  // Initialize servos
  servo1.attach(SERVO1_PIN);
  servo2.attach(SERVO2_PIN);
  servo1.write(0); // Initial position (Gate closed)
  servo2.write(0); // Initial position (Gate closed)

  // Initialize pins
  pinMode(TRIG_PIN_ULTRA1, OUTPUT);
  pinMode(ECHO_PIN_ULTRA1, INPUT);
  pinMode(RED_LED_PIN, OUTPUT);
  pinMode(GREEN_LED_PIN, OUTPUT);
  pinMode(BUZZER_PIN, OUTPUT);

  // Initialize scale
  scale.begin(30, 31);
  scale.set_scale(210);
  scale.tare(); // Reset the scale to 0

  lcd.setCursor(0, 0);
  lcd.print("System Ready");
}

void loop() {
  // Always check the weight and update the LCD and Serial Monitor
  float weight = scale.get_units(1);

  // Trigger ultrasonic 1 (truck approaching the scale)
  distance1 = getUltrasonicDistance(TRIG_PIN_ULTRA1, ECHO_PIN_ULTRA1);
  if (distance1 > 0 && distance1 < 30 && !weightOk) {
    // Open gate to scale
    servo1.write(90);
    delay(100);
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Step on Scale");

    // Tare the scale before starting to weigh
    scale.tare(); // Reset the scale to 0 right before weighing

    // Start weighing
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Weighing...");

    // Continuously update weight for 3 seconds
    unsigned long weighStartTime = millis();
    while (millis() - weighStartTime < 3000) {
      weight = scale.get_units(1);
      lcd.setCursor(0, 1);
      lcd.print("Weight: ");
      lcd.print(weight);
      lcd.print("g  "); // Add some spaces to clear previous characters

      // Update distance1 to see if the truck has moved
      distance1 = getUltrasonicDistance(TRIG_PIN_ULTRA1, ECHO_PIN_ULTRA1);
      delay(100); // Small delay for smooth updating
    }
    

    // Check the weight after the truck has moved off the scale
    weight = scale.get_units(1);
    if (weight <= weightThreshold && digitalRead(32) == 0) 
    {
      // Green light and allow to proceed
      digitalWrite(GREEN_LED_PIN, HIGH);
      digitalWrite(RED_LED_PIN, LOW);

      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Weight OK");

      digitalWrite(BUZZER_PIN, HIGH);
      delay(500);
      digitalWrite(BUZZER_PIN, LOW);
      delay(2000);

      servo2.write(90);  // Allow passing
      delay(3000);
      resetSystem();
    } 
    if (weight > weightThreshold) 
    {
      // Red light and prevent passing
      digitalWrite(GREEN_LED_PIN, LOW);
      digitalWrite(RED_LED_PIN, HIGH);

      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Weight Exceeded");

      digitalWrite(BUZZER_PIN, HIGH);
      delay(500);
      digitalWrite(BUZZER_PIN, LOW);
      servo2.write(90);
      delay(3000);

      resetSystem(); // Reset and prepare for next vehicle
    }
  }

}

long getUltrasonicDistance(int trigPin, int echoPin) {
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);

  long duration = pulseIn(echoPin, HIGH);
  return duration * 0.034 / 2;
}

void resetSystem() {
  servo1.write(0); // Close gate to scale
  servo2.write(0); // Close gate to country
  digitalWrite(RED_LED_PIN, LOW);
  digitalWrite(GREEN_LED_PIN, LOW);
  digitalWrite(BUZZER_PIN, LOW);
  digitalWrite(BUZZER_PIN, HIGH);
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("System Ready");
  delay(500);
  digitalWrite(BUZZER_PIN, LOW);
  // Let the system be ready again for new truck detection
}
